@@ -27,8 +27,8 @@ import six
 
 from . import Adapter
 from .. import errors
-from ..makefile import MakeFile
-
+from .._compat import IS_ABOVE_OPENSSL10
+from ..makefile import StreamReader, StreamWriter
 
 if six.PY3:
     generic_socket_error = OSError
@@ -43,9 +43,9 @@ def _assert_ssl_exc_contains(exc, *msgs):
     if len(msgs) < 1:
         raise TypeError(
             '_assert_ssl_exc_contains() requires '
-            'at least one message to be passed.'
+            'at least one message to be passed.',
         )
-    err_msg_lower = exc.args[1].lower()
+    err_msg_lower = str(exc).lower()
     return any(m.lower() in err_msg_lower for m in msgs)
 
 
@@ -67,19 +67,36 @@ class BuiltinSSLAdapter(Adapter):
     ciphers = None
     """The ciphers list of SSL."""
 
+    CERT_KEY_TO_ENV = {
+        'subject': 'SSL_CLIENT_S_DN',
+        'issuer': 'SSL_CLIENT_I_DN',
+    }
+
+    CERT_KEY_TO_LDAP_CODE = {
+        'countryName': 'C',
+        'stateOrProvinceName': 'ST',
+        'localityName': 'L',
+        'organizationName': 'O',
+        'organizationalUnitName': 'OU',
+        'commonName': 'CN',
+        'emailAddress': 'Email',
+    }
+
     def __init__(
             self, certificate, private_key, certificate_chain=None,
-            ciphers=None):
+            ciphers=None,
+    ):
         """Set up context in addition to base class properties if available."""
         if ssl is None:
             raise ImportError('You must install the ssl module to use HTTPS.')
 
         super(BuiltinSSLAdapter, self).__init__(
-            certificate, private_key, certificate_chain, ciphers)
+            certificate, private_key, certificate_chain, ciphers,
+        )
 
         self.context = ssl.create_default_context(
             purpose=ssl.Purpose.CLIENT_AUTH,
-            cafile=certificate_chain
+            cafile=certificate_chain,
         )
         self.context.load_cert_chain(certificate, private_key)
         if self.ciphers is not None:
@@ -117,6 +134,7 @@ class BuiltinSSLAdapter(Adapter):
                     'wrong version number',
                     'no shared cipher', 'certificate unknown',
                     'ccs received early',
+                    'certificate verify failed',  # client cert w/o trusted CA
                 )
                 if _assert_ssl_exc_contains(ex, *_block_errors):
                     # Accepted error, let's pass
@@ -130,15 +148,17 @@ class BuiltinSSLAdapter(Adapter):
         except generic_socket_error as exc:
             """It is unclear why exactly this happens.
 
-            It's reproducible only under Python 2 with openssl>1.0 and stdlib
-            ``ssl`` wrapper, and only with CherryPy.
-            So it looks like some healthcheck tries to connect to this socket
-            during startup (from the same process).
+            It's reproducible only with openssl>1.0 and stdlib ``ssl`` wrapper.
+            In CherryPy it's triggered by Checker plugin, which connects
+            to the app listening to the socket port in TLS mode via plain
+            HTTP during startup (from the same process).
 
 
             Ref: https://github.com/cherrypy/cherrypy/issues/1618
             """
-            if six.PY2 and exc.args == (0, 'Error'):
+            is_error0 = exc.args == (0, 'Error')
+
+            if is_error0 and IS_ABOVE_OPENSSL10:
                 return EMPTY_RESULT
             raise
         return s, self.get_environ(s)
@@ -151,12 +171,40 @@ class BuiltinSSLAdapter(Adapter):
             'wsgi.url_scheme': 'https',
             'HTTPS': 'on',
             'SSL_PROTOCOL': cipher[1],
-            'SSL_CIPHER': cipher[0]
+            'SSL_CIPHER': cipher[0],
             # SSL_VERSION_INTERFACE     string  The mod_ssl program version
             # SSL_VERSION_LIBRARY   string  The OpenSSL program version
         }
+
+        if self.context and self.context.verify_mode != ssl.CERT_NONE:
+            client_cert = sock.getpeercert()
+            if client_cert:
+                for cert_key, env_var in self.CERT_KEY_TO_ENV.items():
+                    ssl_environ.update(
+                        self.env_dn_dict(env_var, client_cert.get(cert_key)),
+                    )
+
         return ssl_environ
 
+    def env_dn_dict(self, env_prefix, cert_value):
+        """Return a dict of WSGI environment variables for a client cert DN.
+
+        E.g. SSL_CLIENT_S_DN_CN, SSL_CLIENT_S_DN_C, etc.
+        See SSL_CLIENT_S_DN_x509 at
+        https://httpd.apache.org/docs/2.4/mod/mod_ssl.html#envvars.
+        """
+        if not cert_value:
+            return {}
+
+        env = {}
+        for rdn in cert_value:
+            for attr_name, val in rdn:
+                attr_code = self.CERT_KEY_TO_LDAP_CODE.get(attr_name)
+                if attr_code:
+                    env['%s_%s' % (env_prefix, attr_code)] = val
+        return env
+
     def makefile(self, sock, mode='r', bufsize=DEFAULT_BUFFER_SIZE):
         """Return socket file object."""
-        return MakeFile(sock, mode, bufsize)
+        cls = StreamReader if 'r' in mode else StreamWriter
+        return cls(sock, mode, bufsize)
